/*
 * K-scope
 * Copyright 2012-2013 RIKEN, Japan
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.riken.kscope.profiler.dprof;

/**
 * DPROF:コストライン情報クラス
 * @author riken
 */
public class LineInfo {
    private float sampNum;
    private int lineNo;
    private int symbolIndex;
    private int fileIndex;

    /**
     * サンプリング回数を取得する
     * @return サンプリング回数
     */
    public float getSampNum() {
        return sampNum;
    }

    /**
     * ライン番号を取得する
     * @return ライン番号
     */
    public int getLineNo() {
        return lineNo;
    }

    /**
     * シンボルのindexを取得する
     * @return シンボルのindex
     */
    public int getSymbolIndex() {
        return symbolIndex;
    }

    /**
     * ファイルのindexを取得する
     * @return ファイルのindex
     */
    public int getFileIndex() {
        return fileIndex;
    }

    /**
     * サンプリング回数を設定する
     * @param sampNum
     *            設定するサンプリング回数
     */
    public void setSampNum(float sampNum) {
        this.sampNum = sampNum;
    }

    /**
     * ライン番号を設定する
     * @param lineNo
     *            設定するライン番号
     */
    public void setLineNo(int lineNo) {
        this.lineNo = lineNo;
    }

    /**
     * シンボルのindexを設定する
     * @param symbolIndex
     *            設定するシンボルのindex
     */
    public void setSymbolIndex(int symbolIndex) {
        this.symbolIndex = symbolIndex;
    }

    /**
     * ファイルのindexを設定する
     * @param fileIndex
     *            設定するファイルのindex
     */
    public void setFileIndex(int fileIndex) {
        this.fileIndex = fileIndex;
    }

}
